//   Copyright 2007, 2008 Nicolas Dubuit
//
//     This file is part of Immix.
//
//     Immix is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <QList>
#include <QUrl>
#include <QMimeData>

#include "initialwindow.h"
#include "immix.h"

InitialWindow::InitialWindow (ImmixWin* immixW, const QString& file, QWidget* parent)
    : QWidget (parent)
{
    imw = immixW ;

// Label.
    lab = new QLabel (tr("Drop files here to start alignment, or click \"Select files \"."), this) ;

// "Select files" button.
    pb = new QPushButton (tr("Select files..."), this);
    connect (pb, SIGNAL(clicked()), this, SLOT(getFiles()) ) ;

// Renderer.
    renderer = new QSvgRenderer (file, this) ;

// Let's accept drops.
    setAcceptDrops(true);
}

void InitialWindow::resizeEvent (QResizeEvent* event)
{
    int w = event->size().width() ;
    int h = event->size().height() ;

    lab->move ( int(w*0.1), int(h*0.1) ) ;
    pb->move  ( int(w*0.65), int(h*0.75) ) ;
}
  
void InitialWindow::paintEvent(QPaintEvent *)
{
// Draw SVG.
    QPainter painter (this) ;
    renderer->setViewBox (QRect(0,500,550,550)) ;
    renderer->render(&painter, painter.window());

// Make it almost white.
    painter.setBrush (QColor (255,255,255,150)) ;
    painter.setPen (Qt::NoPen) ;
    painter.drawRect (painter.window()) ;

/*    QRect vb = renderer->viewBox() ;
    std::cout << "viewBox : " << vb.width() << "x" << vb.height() << " at " 
              << vb.x() << "," << vb.y() << std::endl ;
    std::cout << "sz : " << size().width() << "x" << size().height() << std::endl ;*/
}

void InitialWindow::dragEnterEvent(QDragEnterEvent *event)
{
   if (event->mimeData()->hasUrls())
     if ( event->proposedAction() == Qt::MoveAction )
     {
        event->setDropAction (Qt::CopyAction) ;
        event->accept();
     }
     else
        event->acceptProposedAction();
}

void InitialWindow::dragMoveEvent(QDragMoveEvent *event)
{
   if (event->mimeData()->hasUrls())
    if ( event->proposedAction() == Qt::MoveAction )
    {
        event->setDropAction (Qt::CopyAction) ;
        event->accept();
    }
    else
        event->acceptProposedAction();
}

void InitialWindow::dropEvent(QDropEvent *event)
{
    const QMimeData *mimeData = event->mimeData();

    if (!mimeData->hasUrls())
        return ;

// Get path list from URL list, verify files are readable.
    QStringList pathList ;
    QList<QUrl> urlList = mimeData->urls();
    foreach (QUrl url, urlList)
    {
        if ( ! QFileInfo (url.toLocalFile()).isReadable() )
            std::cerr << "Can't read file : \"" << url.path().toStdString() 
                      << "\" ; skipped."<< std::endl ;
        else
            pathList.append (url.toLocalFile());
    }

// Start main interface with given files.
    imw->SetFileList (pathList) ;

// We don't accept "move" actions since the data is not copied.
    if ( event->proposedAction() == Qt::MoveAction )
    {
        event->setDropAction (Qt::CopyAction) ;
        event->accept();
    }
    else
        event->acceptProposedAction();
 }




void InitialWindow::getFiles(void)
{
    QStringList files ;
    while (files.size() < 2)
    {
        files = QFileDialog::getOpenFileNames(this,
                            tr("Select the images to align"),
                            QString(), tr("Images (*.png *.xpm *.jpg)") );
        if (files.size() < 2)
          if ( QMessageBox::Abort == QMessageBox::critical (this, "immix", 
                                       tr("You must select at least 2 files to align and merge."), 
                                       QMessageBox::Retry | QMessageBox::Abort, QMessageBox::Retry) )
          {
            return ;
          }
    }

    imw->SetFileList (files) ;
}
